###############   PSQF 6271 Longitudinal MLM Example 4 using R   #############

# Set width of output and  number of digits to be printed
options("width"=120, "digits"=8) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with random effects and/or R cov structures


################################################################################
#####               BEGIN EXAMPLE 4 DATA MANIUPLATION                      #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6271/PSQF6271_Example4/"
filename = "SAS_Example4.sas7bdat"
setwd(dir=filesave)

# Import Example 4 seven-occasion stacked data with labels
Example4 = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example4 = as.data.frame(Example4)
# Labels can be used with gls, so I will use them here
# Sort data by person (needed for correct RCOV matrix)
Example4 = sort_asc(Example4,subid,week)
# It insists I make a new week variable starting at 1, not 2 (insert eye-roll)
Example4$week1=Example4$week-1

################################################################################
#####                   BEGIN EXAMPLE 4 MODELS                             #####
################################################################################

# Open output text file
txtStart(file=paste0(filesave,"R_Example4_Output.txt"))

############################ R-ONLY MODELS ##############################

print("R GLS n-order Unstructured R-only Model")
UN = gls(data=Example4, method="REML", model=severity~1, 
         correlation=corSymm(form=~as.numeric(week1)|subid),
         weights=varIdent(form=~1|week1)) 
print("Show results using incorrect DDF, with total leftover variance")
print("Total variance per occasion is created using SD multiplier")
summary(UN); summary(UN)$sigma^2 
print("Show R and RCORR matrices for first person") 
getVarCov(UN, individual="100", type="marginal")
corMatrix(UN$modelStruct$corStruct)[[6]] 
txtComment(" ") # insert blank space in output


print("R GLS Compound Symmetry R-only Model")
CS = gls(data=Example4, method="REML", model=severity~1, 
				 correlation=corCompSymm(form=~1|subid))
print("Show results using incorrect DDF"); summary(CS)
print("Show R and RCORR matrices for first person") 
getVarCov(CS, individual="100", type="marginal"); 
corMatrix(CS$modelStruct$corStruct)[[6]] 
print("LRT: Is CS worse than UN?"); anova(UN,CS) 
txtComment(" ") # insert blank space in output


print("R GLS Auto-Regressive R-only Model")
AR = gls(data=Example4, method="REML", model=severity~1, 
				  correlation=(corAR1(form=~as.numeric(week1)|subid)))
# correlation=(corCAR1(form=~as.numeric(week1)|subid)) for unbalanced time
print("Show results using incorrect DDF"); summary(AR); 
print("Show R and RCORR matrices for first person") 
getVarCov(AR, individual="100", type="marginal");  
corMatrix(AR$modelStruct$corStruct)[[6]] 
print("LRT: Is AR worse than UN?"); anova(UN,AR) 
txtComment(" ") # insert blank space in output


# Toeplitz is not a pre-defined structure in R GLS


########################## R & G COMBINED MODELS ###########################

# Cannot see how to fit UN6 R matrix in R lme (defaults to UN7)


print("R LME Random Intercept + Diagonal R Model")
RI = lme(data=Example4, method="REML", severity~1, random=~1|subid,
         correlation=NULL) # No correlation in R matrix
print("Show results using incorrect DDF"); summary(RI); 
print("Show G, R, and V matrices for first person") 
G=getVarCov(RI, individual="100", type="random.effects"); G 
R=getVarCov(RI, individual="100", type="conditional"); R    
V=getVarCov(RI, individual="100", type="marginal"); V      
ICC=(V[[1]][2,1])/(V[[1]][1,1]); print("Show ICC"); ICC
print("LRT: Is RI worse than UN?"); anova(UN,RI) 
txtComment(" ") # insert blank space in output


print("R LME Random Intercept + Auto-Regressive R Model")
ARRI = lme(data=Example4, method="REML", severity~1, random=~1|subid,
           correlation=(corAR1(form=~as.numeric(week1)|subid)))
# correlation=(corCAR1(form=~as.numeric(week1)|subid)) for unbalanced time
print("Show results using incorrect DDF"); summary(ARRI)
print("Show G, R, RCORR, and V matrices for first person") 
G=getVarCov(ARRI, individual="100", type="random.effects"); G 
R=getVarCov(ARRI, individual="100", type="conditional"); R
RCORR=corMatrix(ARRI$modelStruct$corStruct)[[6]]; RCORR
V=getVarCov(ARRI, individual="100", type="marginal"); V
print("LRT: Is ARRI better than AR?"); anova(ARRI,AR) 
print("LRT: Is ARRI worse than UN?"); anova(UN,ARRI) 
txtComment(" ") # insert blank space in output


# Toeplitz is not a pre-defined structure in R lme


# Close output text file
txtStop()
