* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS LISTING;

***********************************************************************************;
*******  MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR CHAPTER 4 EXAMPLE     *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=,FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

***********************************************************************************;
*******     BEGIN DATA MANIPULATION OF EXAMPLE 4 SEVEN-OCCASION DATA        *******;
*******               CHANGE "filesave" to your directory                   *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
%LET filesave = C:\Dropbox\22_PSQF6271\PSQF6271_Example4;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";
    
* Import Example 4 seven-occasion stacked data (not from textbook);
DATA work.Example4; SET filesave.SAS_Example4; RUN;

***********************************************************************************;
*******                BEGIN EXAAMPLE 4 SEVEN-OCCASION MODELS               *******;
***********************************************************************************;

* Open output directory to save results to;
ODS RTF FILE="&filesave.\SAS_Example4_Output.rtf" STYLE=HTMLBlue STARTPAGE=NO;

*********************** R-ONLY MODELS ************************;

TITLE1 "SAS n-order Unstructured (UN) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity =  / SOLUTION DDFM=Satterthwaite;
     REPEATED week / TYPE=UN R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitUN; * Save for LRT;
RUN; TITLE1;

TITLE1 "SAS Compound Symmetry (CS) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     REPEATED week / TYPE=CS R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitCS; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitCS, FitMore=FitUN);
 
TITLE1 "SAS Auto-Regressive (AR) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     REPEATED week / TYPE=AR(1) R RCORR SUBJECT=subid; 
* Use this version for unbalanced time;
	*REPEATED week / TYPE=SP(POW)(week) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitAR; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitAR, FitMore=FitUN);

TITLE1 "SAS Toeplitz (n=7 bands) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     REPEATED week / TYPE=TOEP(7) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep7; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitToep7, FitMore=FitUN);

TITLE1 "SAS Heterogeneous Toeplitz (n=7 bands) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     REPEATED week / TYPE=TOEPH(7) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep7H; * Save for LRT;
RUN; TITLE1;
* Model comparison: did het variance help?;
  %FitTest(FitFewer=FitToep7, FitMore=FitToep7H);
* Model comparison: now not worse than UN?;
  %FitTest(FitFewer=FitToep7H, FitMore=FitUN);


********************** G & R COMBINED MODELS **************************;

TITLE1 "SAS Random Intercept + Unstructured(n-1) R Model";
TITLE2 "Equal to n-Order Unstructured R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid;
     REPEATED week / TYPE=UN(6) R RCORR SUBJECT=subid; 
RUN; TITLE1; TITLE2;

TITLE1 "SAS Random Intercept + Diagonal R Model";
TITLE2 "Equal to Compound Symmetry R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid;
     REPEATED week / TYPE=VC R RCORR SUBJECT=subid; 
RUN; TITLE1; TITLE2;

TITLE1 "SAS Random Intercept + Auto-Regressive R Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=AR(1) R RCORR SUBJECT=subid; 
* Use this version for unbalanced time;
	*REPEATED week / TYPE=SP(POW)(week) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitARRI; * Save for LRT;
RUN; TITLE1;
* Model comparison: did random intercept help?;
  %FitTest(FitFewer=FitAR, FitMore=FitARRI);
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitARRI, FitMore=FitUN);

TITLE1 "SAS Random Intercept + 5-Lag Toeplitz6 R Model";
TITLE2 "Equal to Toeplitz (n=7 bands) R-only Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=TOEP(6) R RCORR SUBJECT=subid; 
RUN; TITLE1; TITLE2;

TITLE1 "SAS Random Intercept + 4-Lag Toeplitz5 R Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=TOEP(5) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep5RI; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitToep5RI, FitMore=FitUN);

TITLE1 "SAS Random Intercept + 3-Lag Toeplitz4 R Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=TOEP(4) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep4RI; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitToep4RI, FitMore=FitUN);

TITLE1 "SAS Random Intercept + 2-Lag Toeplitz3 R Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=TOEP(3) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep3RI; * Save for LRT;
RUN; TITLE1;
* Model comparison: not worse than UN?;
  %FitTest(FitFewer=FitToep3RI, FitMore=FitUN);

TITLE1 "SAS Random Intercept + 3-Lag Heterogeneous Toeplitz4 R Model";
PROC MIXED DATA=work.Example4 COVTEST NOCLPRINT IC METHOD=REML;
     CLASS subid week;
     MODEL severity = / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN V VCORR G SUBJECT=subid; 
     REPEATED week / TYPE=TOEPH(4) R RCORR SUBJECT=subid; 
     ODS OUTPUT InfoCrit=FitToep4HRI; * Save for LRT;
RUN; TITLE1;
* Model comparison: did het variance help?;
  %FitTest(FitFewer=FitToep4RI, FitMore=FitToep4HRI);
* Model comparison: now not worse than UN?;
  %FitTest(FitFewer=FitToep4HRI, FitMore=FitUN);

 
****** END EXAMPLE 4 MODELS ******;

* Close output;
ODS RTF CLOSE;

