// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

********************************************************************************
*****    EXAMPLE 4: ALTERNATIVE COVARIANCE STRUCTURE MODELS               ******
*******************************************************************************.

// Defining global variable for file location to be replaced in code below
   cd "C:\Dropbox\22_PSQF6271\PSQF6271_Example4"
   use "STATA_Example4.dta", clear
   gen week=Week

// Save results to separate file
log using STATA_Example4_Output.log, replace

********************** R-ONLY MODELS ************************
 
display "STATA n-order Unstructured R-only Model"
mixed severity ,  || subid:  , noconstant              /// 
         variance reml residuals(unstructured,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat wcorrelation, covariance  // R matrix
      estat wcorrelation              // RCORR matrix
      estat ic, n(124)                // AIC and BIC
      estimates store FitUN           // Save for LRT

display "STATA Compound Symmetry (CS) R-only Model"
mixed severity ,  || subid:  , noconstant              /// 
         variance reml residuals(exchangeable,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat wcorrelation, covariance  // R matrix
      estat wcorrelation              // RCORR matrix
      estat ic, n(124)                // AIC and BIC
      estimates store FitCS           // Save for LRT
      lrtest FitUN FitCS              // Worse than UN?
     
display "STATA Auto-Regressive (AR) R-only Model"
mixed severity ,  || subid:  , noconstant       /// 
        variance reml residuals(ar1,t(week))    ///
        dfmethod(satterthwaite) dftable(pvalue)
      estat wcorrelation, covariance  // R matrix
      estat wcorrelation              // RCORR matrix
      estat ic, n(124)                // AIC and BIC
      estimates store FitAR           // Save for LRT
      lrtest FitUN FitAR              // Worse than UN?
     
display "STATA Toeplitz (n=7 bands) R-only Model"
mixed severity ,  || subid:  , noconstant           /// 
         variance reml residuals(toeplitz6,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat wcorrelation, covariance  // R matrix
      estat wcorrelation              // RCORR matrix
      estat ic, n(124)                // AIC and BIC
      estimates store FitToep7        // Save for LRT
      lrtest FitUN FitToep7           // Worse than UN?
     
// STATA Heterogeneous Toeplitz (n=7 bands) R-only Model 
// is not possible as far as I can tell
 
********************** G & R COMBINED MODELS **********************
 
display "STATA Random Intercept + Unstructured(n-1) R Model"
display "Equal to n-Order Unstructured R-only Model"
mixed severity ,  || subid:  ,                    /// 
         variance reml residuals(banded6,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat recovariance, relevel(subid) // G matrix
      estat wcorrelation, covariance     // V matrix
      estat wcorrelation                 // VCORR matrix
      estat ic, n(124)                   // AIC and BIC
     
display "STATA Random Intercept + Diagonal R Model"
display "Equal to Compound Symmetry R-only Model"
mixed severity ,  || subid:  ,                       /// 
        variance reml residuals(independent,t(week)) ///
        dfmethod(satterthwaite) dftable(pvalue)
      estat recovariance, relevel(subid) // G matrix
      estat wcorrelation, covariance     // V matrix
      estat wcorrelation                 // VCORR matrix
      estat ic, n(124)                   // AIC and BIC 
      estat icc                          // ICC 
     
display "STATA Random Intercept + Auto-Regressive R Model"
mixed severity ,  || subid:  ,                /// 
         variance reml residuals(ar1,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat recovariance, relevel(subid) // G matrix
      estat wcorrelation, covariance     // V matrix
      estat wcorrelation                 // VCORR matrix
      estat ic, n(124)                   // AIC and BIC 
      estimates store FitARRI            // Save for LRT
      lrtest FitARRI FitAR               // Better than AR?
      lrtest FitUN FitARRI               // Worse than UN?

display "STATA Random Intercept + 5-lag Toeplitz6 R Model"
display "Equal to Toeplitz (n=7 bands) R-only Model"
mixed severity ,  || subid:  ,                      /// 
         variance reml residuals(toeplitz5,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat recovariance, relevel(subid) // G matrix
      estat wcorrelation, covariance     // V matrix
      estat wcorrelation,                // VCORR matrix
      estat ic, n(124),                  // AIC and BIC 

display "STATA Random Intercept + 4-lag Toeplitz5 R Model"
mixed severity ,  || subid:  ,                      /// 
         variance reml residuals(toeplitz4,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
      estat recovariance, relevel(subid) // G matrix
      estat wcorrelation, covariance     // V matrix
      estat wcorrelation                 // VCORR matrix
      estat ic, n(124)                   // AIC and BIC 
      estimates store FitToep5RI         // Save for LRT
      lrtest FitUN FitToep5RI            // Worse than UN?
     
display "STATA Random Intercept + 3-lag Toeplitz4 R Model"
mixed severity ,  || subid:  ,                      /// 
         variance reml residuals(toeplitz3,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
     estat recovariance, relevel(subid) // G matrix
     estat wcorrelation, covariance     // V matrix
     estat wcorrelation                 // VCORR matrix
     estat ic, n(124)                   // AIC and BIC 
     estimates store FitToep4RI         // Save for LRT
     lrtest FitUN FitToep4RI            // Worse than UN?
     
display "STATA Random Intercept + 2-lag Toeplitz3 R Model"
mixed severity ,  || subid:  ,                      /// 
         variance reml residuals(toeplitz2,t(week)) ///
         dfmethod(satterthwaite) dftable(pvalue)
     estat recovariance, relevel(subid) // G matrix
     estat wcorrelation, covariance     // V matrix
     estat wcorrelation                 // VCORR matrix
     estat ic, n(124)                   // AIC and BIC 
     estimates store FitToep3RI         // Save for LRT
     lrtest FitUN FitToep3RI            // Worse than UN?
     
// STATA Heterogeneous Toeplitz models are 
// not possible as far as I can tell
     
// Close log   
 log close
